#                                                         -*- Perl -*-
# Copyright (C) 2001-2007  Kazuhiko  <kazuhiko@fdiary.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

%replace_list = (
  '"' => '&#x0251;',
  '#' => '&#x025b;',
  '$' => '&#x0254;',
  '%' => '&oelig;',
  "'" => '&#x0259;',
  '+' => '&#x0251;',
  '0' => '&#x0292;',
  '1' => '&atilde;',
  '2' => '&#x0251;&#x0303;',
  '3' => '&#x025b;&#x0303;',
  '4' => '&#x0254;&#x0303;',
  '5' => '&oelig;&#x0303;',
  '6' => '&#x0283;',
  '7' => '&#x014b;',
  '8' => '&#x03b8;',
  '9' => '&#x00f0;',
  '<' => '&#x025a;&#0300;',
  '=' => '&#x025a;',
  '>' => '&#x025a;&#0301;',
  '?' => '&oslash;',
  'G' => '&#x025b;&#x0300;',
  'M' => '&#x0272;',
  '_' => '&#x0265;'
);

sub fpw_add_entry_entity_reference {
    my ($obj, $item) = @_;
    my $header = '';

    foreach $elem (split /(&.+?;)/, $item) {
	if ($elem =~ /^&(.).*;$/) {
	    $header .= $1;
	} else {
	    $header .= $elem;
	}
    }
    $obj->add_entry($header, $heading_position, $text_position)
	|| fpw_error($obj);
    return $header;
}

sub fpw_add_text_royal {
    my ($obj, $item) = @_;
    $item =~ s/<span class="gosi">(.+?)<\/span>/\1/g;
    foreach $elem (split /(<span class=".+?">.+?<\/span>)/, $item) {
	if ($elem =~ /<span class="hatuon">(.+?)<\/span>/) {
	    foreach $char (split(//, $1)) {
		if (exists($replace_list{$char})) {
		    $obj->add_text($replace_list{$char}) ||
			die "$PROGRAM_NAME: " . $obj->error_message() . "\n";
		} else {
		    $obj->add_text($char) ||
			die "$PROGRAM_NAME: " . $obj->error_message() . "\n";
		}
	    }
	} else {
	    foreach (split(/((?:<\/?(?:b|i|sup|sub)>)|<span class="jump" onClick="openNewWin\('..\/..\/..\/zuhan\/.+?','ZUHAN'\)">|<span class="jump" onClick="jump\('.+?',''\)">.+?<\/span>)/, $elem)) {
		if ($_ eq "<b>") {
		    $obj->add_emphasis_start() || fpw_error($obj);
		} elsif ($_ eq "</b>") {
		    $obj->add_emphasis_end() || fpw_error($obj);
		} elsif ($_ eq "<i>") {
		    $obj->add_font_start('italic') || fpw_error($obj);
		} elsif ($_ eq "</i>") {
		    $obj->add_font_end() || fpw_error($obj);
		} elsif ($_ eq "<sup>") {
		    $obj->add_superscript_start || fpw_error($obj);
		} elsif ($_ eq "</sup>") {
		    $obj->add_superscript_end() || fpw_error($obj);
		} elsif ($_ eq "<sub>") {
		    $obj->add_subscript_start || fpw_error($obj);
		} elsif ($_ eq "</sub>") {
		    $obj->add_subscript_end() || fpw_error($obj);
		} elsif (/<span class="jump" onClick="openNewWin\('..\/..\/..\/zuhan\/(.+?).pdf','ZUHAN'\)">/) {
		    if (!$obj->add_color_graphic_start($1)
			|| !$obj->add_color_graphic_end()) {
			fpw_error($obj);
		    }
		} elsif (/<span class="jump" onClick="jump\('(.+?)',''\)">(.+?)<\/span>/) {
		    $ref = $1;
		    $target = $2;
		    $obj->add_reference_start() || fpw_error($obj);
		    fpw_add_text_royal($obj, $target);
		    $obj->add_reference_end($ref) || fpw_error($obj);
		} else {
		    s/<[^>]+>/ /g;
		    s/ +/ /g;
		    $obj->add_text($_) ||
			die "$PROGRAM_NAME: " . $obj->error_message() . "\n";
		}
	    }
	}
    }
}

sub fpw_add_entry {
    my ($obj, $item) = @_;

    $obj->add_entry($item, $heading_position, 'head',
		    $text_position, 'text') || fpw_error($obj);
    return $item;
}

sub fpw_add_text {
    my ($obj, $item) = @_;
    $obj->add_text($item) || fpw_error($obj);
    return $item;
}

sub fpw_add_newline {
    my $obj = $_[0];
    $obj->add_newline() || fpw_error($obj);
}

sub fpw_add_keyword_latin1 {
    my ($obj, $item) = @_;
    $obj->add_keyword_start() || fpw_error($obj);
    fpw_add_text_latin1($obj, $item);
    if (!$obj->add_keyword_end()
	|| !$obj->add_newline()
	|| !$obj->add_indent_level(2)) {
	fpw_error($obj);
    }
}

sub fpw_add_keyword {
    my ($obj, $item) = @_;
    $obj->add_keyword_start() || fpw_error($obj);
    fpw_add_text($obj, $item);
    if (!$obj->add_keyword_end()
	|| !$obj->add_newline()
	|| !$obj->add_indent_level(2)) {
	fpw_error($obj);
    }
}

sub fpw_add_entry_tag {
    my ($obj, $item) = @_;
    $obj->add_entry_tag(fpw_tag($item)) || fpw_error($obj);
}

sub fpw_add_reference_start {
    my $obj = $_[0];
    $obj->add_reference_start() || fpw_error($obj);
}

sub fpw_add_reference_end {
    my ($obj, $item) = @_;
    $obj->add_reference_end(fpw_tag($item)) || fpw_error($obj);
}

sub fpw_add_new_entry {
    my $obj = $_[0];
    $obj->new_entry() || fpw_error($obj);
}

sub fpw_add_menu {
    my ($obj, $item) = @_;
    if (!$obj->add_reference_start()
	|| !$obj->add_text($item)
	|| !$obj->add_reference_end(fpw_tag($item))
	|| !$obj->add_newline()) {
	fpw_error($obj);
    }
}

sub fpw_add_menu_entry {
    my ($obj, $item) = @_;
    if (!$obj->new_context()
	|| !$obj->add_tag(fpw_tag($item))) {
	fpw_error($obj);
    }
}

sub fpw_add_color_graphic_start {
    my ($obj, $item) = @_;
    $obj->add_color_graphic_start($item) || fpw_error($obj);
}

sub fpw_add_color_graphic_end {
    my $obj = $_[0];
    $obj->add_color_graphic_end() || fpw_error($obj);
}

sub fpw_add_color_jpeg_start {
    my ($obj, $item) = @_;
    $obj->add_color_jpeg_start($item) || fpw_error($obj);
}

sub fpw_add_color_jpeg_end {
    my $obj = $_[0];
    $obj->add_color_jpeg_end() || fpw_error($obj);
}

sub fpw_error {
  my($obj) = $_[0];
  die "$PROGRAM_NAME: " . $obj->error_message() . "\n";
}

sub fpw_tag {
    my $item = $_[0];
    return "T" . unpack "H*", $item;
}

sub fpw_z2h {
  my($obj) = jcode($_[0]);
  $obj = $obj->tr("ɡǡʡˡܡݡ","!\"#\$%&'()*+,-./");
  $obj = $obj->tr("䡩ΡϡơСá",":;<=>?[\]^_`{|}");
  $obj = $obj->tr("-","0-9");
  $obj = $obj->tr("-","A-Z");
  $obj = $obj->tr("-","a-z");
  return $obj;
}

1;
